@@ -1,13 +1,11 @@
-#!perl
 package IO::Socket::Telnet;
 use strict;
 use warnings;
-use parent 'IO::Socket::INET';
-use Class::Method::Modifiers;
+use base 'IO::Socket::INET';
 
-around new => sub
-{
-    my $orig = shift;
+our $VERSION = '0.03';
+
+sub new {
     my $class = shift;
     my %args = @_;
 
@@ -15,38 +13,40 @@ around new => sub
         if exists $args{PeerAddr}
         || exists $args{PeerHost};
 
-    my $self = $class->$orig(%args);
+    my $self = $class->SUPER::new(%args);
+    return undef if !defined($self);
 
     ${*$self}{telnet_mode} = 'normal';
     ${*$self}{telnet_sb_buffer} = '';
 
     return $self;
+}
+
+sub recv {
+    my $self = shift;
+
+    $self->SUPER::recv(@_);
+    $_[0] = $self->_parse($_[0]);
 };
 
-after recv => sub
-{
-    $_[1] = $_[0]->_parse($_[1]);
-};
-
-sub telnet_simple_callback
-{
+sub telnet_simple_callback {
     my $self = shift;
     ${*$self}{telnet_simple_cb} = $_[0] if @_;
     ${*$self}{telnet_simple_cb};
 }
 
-sub telnet_complex_callback
-{
+sub telnet_complex_callback {
     my $self = shift;
     ${*$self}{telnet_complex_cb} = $_[0] if @_;
     ${*$self}{telnet_complex_cb};
 }
 
-our @options = qw(BINARY ECHO RCP SGA NAMS STATUS TM RCTE NAOL NAOP NAOCRD
-                  NAOHTS NAOHTD NAOFFD NAOVTS NAOVTD NAOLFD XASCII LOGOUT BM DET
-                  SUPDUP SUPDUPOUTPUT SNDLOC TTYPE EOR TUID OUTMRK TTYLOC
-                  VT3270REGIME X3PAD NAWS TSPEED LFLOW LINEMODE XDISPLOC
-                  OLD_ENVIRON AUTHENTICATION ENCRYPT NEW_ENVIRON);
+our @options = qw(
+    BINARY ECHO RCP SGA NAMS STATUS TM RCTE NAOL NAOP NAOCRD NAOHTS NAOHTD
+    NAOFFD NAOVTS NAOVTD NAOLFD XASCII LOGOUT BM DET SUPDUP SUPDUPOUTPUT SNDLOC
+    TTYPE EOR TUID OUTMRK TTYLOC VT3270REGIME X3PAD NAWS TSPEED LFLOW LINEMODE
+    XDISPLOC OLD_ENVIRON AUTHENTICATION ENCRYPT NEW_ENVIRON
+);
 
 our @meta;
 
@@ -68,31 +68,36 @@ our %meta;
     @meta{ @meta }       = 0 .. @meta;
 }
 
-sub will
-{
+sub will {
     my ($self, $opt) = @_;
-    if (exists $options{$opt}) { $opt = $options{$opt} }
+    if (exists $options{$opt}) {
+        $opt = $options{$opt};
+    }
     $self->send($IAC . $WILL . $opt);
 }
 
-sub wont
-{
+sub wont {
     my ($self, $opt) = @_;
-    if (exists $options{$opt}) { $opt = $options{$opt} }
+    if (exists $options{$opt}) {
+        $opt = $options{$opt};
+    }
     $self->send($IAC . $WONT . $opt);
 }
 
-sub do
-{
+sub do {
     my ($self, $opt) = @_;
-    if (exists $options{$opt}) { $opt = $options{$opt} }
+    if (exists $options{$opt}) {
+        $opt = $options{$opt};
+    }
     $self->send($IAC . $DO . $opt);
 }
 
 sub dont
 {
     my ($self, $opt) = @_;
-    if (exists $options{$opt}) { $opt = $options{$opt} }
+    if (exists $options{$opt}) {
+        $opt = $options{$opt};
+    }
     $self->send($IAC . $DONT . $opt);
 }
 
@@ -121,17 +126,14 @@ sub dont
 #                 to sb mode. if we receive an SE (subneg-end) in this mode,
 #                 perform some kind of action and go back to normal mode
 
-my %dispatch =
-(
-    normal => sub
-    {
+my %dispatch = (
+    normal => sub {
         my ($self, $c) = @_;
         return $c unless $c eq $IAC;
         return (undef, $IAC);
     },
 
-    $IAC => sub
-    {
+    $IAC => sub {
         my ($self, $c) = @_;
         return ($IAC, 'normal') if $c eq $IAC;
         return (undef, $c) if $c eq $DO || $c eq $DONT
@@ -143,34 +145,29 @@ my %dispatch =
         Carp::croak "Invalid telnet stream: ... IAC $c (chr ".chr($c).") ...";
     },
 
-    $DO => sub
-    {
+    $DO => sub {
         my ($self, $c, $m) = @_;
         $self->_telnet_simple_callback($m, $c);
         return (undef, 'normal');
     },
 
-    $SB => sub
-    {
+    $SB => sub {
         my ($self, $c) = @_;
         return (undef, 'sbiac') if $c eq $IAC;
         ${*$self}{telnet_sb_buffer} .= $c;
         return;
     },
 
-    sbiac => sub
-    {
+    sbiac => sub {
         my ($self, $c) = @_;
 
-        if ($c eq $IAC)
-        {
+        if ($c eq $IAC) {
             ${*$self}{telnet_sb_buffer} .= $IAC;
             return (undef, $SB);
         }
 
-        if ($c eq $SE)
-        {
-            $self->_telnet_complex_callback();
+        if ($c eq $SE) {
+            $self->_telnet_complex_callback(${*$self}{telnet_sb_buffer});
             ${*$self}{telnet_sb_buffer} = '';
             return (undef, 'normal');
         }
@@ -184,23 +181,20 @@ my %dispatch =
 $dispatch{$DONT} = $dispatch{$WILL} = $dispatch{$WONT} = $dispatch{$DO};
 
 # this takes the input stream and jams it through the FSM
-sub _parse
-{
+sub _parse {
     my ($self, $in) = @_;
     my $out = '';
 
     # optimization: if we're in normal mode then we can quickly move all the
     # input up to the first IAC into the output buffer.
-    if (${*$self}{telnet_mode} eq 'normal')
-    {
+    if (${*$self}{telnet_mode} eq 'normal') {
         # if there is no IAC then we can skip telnet entirely
         $in =~ s/^([^$IAC]*)//o;
         return $1 if length $in == 0;
         $out = $1;
     }
 
-    for my $c (split '', $in)
-    {
+    for my $c (split '', $in) {
         my ($o, $m)
             = $dispatch{${*$self}{telnet_mode}}
                 ->($self, $c, ${*$self}{telnet_mode});
@@ -213,53 +207,51 @@ sub _parse
 }
 
 # called when we get a full DO/DONT/WILL/WONT
-sub _telnet_simple_callback
-{
+sub _telnet_simple_callback {
     my ($self, $mode, $opt) = @_;
     my $response;
 
-    if (${*$self}{telnet_simple_cb})
-    {{
-        my $wopt = ord $opt;
-        $wopt = $options[$wopt] || $wopt;
-
-        my $wmode = ord $mode;
-        $wmode = $meta[$wmode] || $wmode;
-
-        $response = ${*$self}{telnet_simple_cb}->($self, "$wmode $wopt");
-
-        last if !defined($response);
-
-        if ($response eq "0")
+    if (${*$self}{telnet_simple_cb}) {
         {
-            if ($mode eq $DONT) { $response = $IAC . $WILL . $opt }
-            if ($mode eq $DO)   { $response = $IAC . $WONT . $opt }
-            if ($mode eq $WILL) { $response = $IAC . $DONT . $opt }
-            if ($mode eq $WONT) { $response = $IAC . $DO   . $opt }
-            last;
-        }
+            my $wopt = ord $opt;
+            $wopt = $options[$wopt] || $wopt;
 
-        if ($response eq "1")
-        {
-            if ($mode eq $DO)   { $response = $IAC . $WILL . $opt }
-            if ($mode eq $DONT) { $response = $IAC . $WONT . $opt }
-            if ($mode eq $WONT) { $response = $IAC . $DONT . $opt }
-            if ($mode eq $WILL) { $response = $IAC . $DO   . $opt }
-            last;
-        }
+            my $wmode = ord $mode;
+            $wmode = $meta[$wmode] || $wmode;
 
-        my $r = $response;
-        $r =~ s/'//g; # just in case they said "DON'T" or "WON'T"
+            $response = ${*$self}{telnet_simple_cb}->($self, "$wmode $wopt");
 
-        if ($r eq 'DO' || $r eq 'DONT' || $r eq 'WILL' || $r eq 'WONT')
-        {
-            $r = chr($meta{$r});
-            $response = $IAC . $r . $opt;
+            last if !defined($response);
+
+            if ($response eq "0") {
+                if ($mode eq $DONT) { $response = $IAC . $WILL . $opt }
+                if ($mode eq $DO)   { $response = $IAC . $WONT . $opt }
+                if ($mode eq $WILL) { $response = $IAC . $DONT . $opt }
+                if ($mode eq $WONT) { $response = $IAC . $DO   . $opt }
+                last;
+            }
+
+            if ($response eq "1") {
+                if ($mode eq $DO)   { $response = $IAC . $WILL . $opt }
+                if ($mode eq $DONT) { $response = $IAC . $WONT . $opt }
+                if ($mode eq $WONT) { $response = $IAC . $DONT . $opt }
+                if ($mode eq $WILL) { $response = $IAC . $DO   . $opt }
+                last;
+            }
+
+            my $r = $response;
+            $r =~ s/'//g; # just in case they said "DON'T" or "WON'T"
+
+            if ($r eq 'DO' || $r eq 'DONT' || $r eq 'WILL' || $r eq 'WONT') {
+                $r = chr($meta{$r});
+                $response = $IAC . $r . $opt;
+            }
         }
-    }}
+    }
 
     $response = $self->_reasonable_response($mode, $opt)
         if !defined($response);
+
     $self->send($response);
 }
 
@@ -275,25 +267,18 @@ sub _reasonable_response
     return "";
 }
 
-sub _telnet_complex_callback
-{
+sub _telnet_complex_callback {
     my ($self, $sb) = @_;
     ${*$self}{telnet_complex_cb} or return;
     ${*$self}{telnet_complex_cb}->($self, $sb);
 }
 
+__END__
+
 =head1 NAME
 
 IO::Socket::Telnet - transparent telnet negotiation for IO::Socket::INET
 
-=head1 VERSION
-
-Version 0.02 released 04 Nov 07
-
-=cut
-
-our $VERSION = '0.02';
-
 =head1 SYNOPSIS
 
     use IO::Socket::Telnet;
@@ -427,52 +412,14 @@ L<Net::Telnet>, L<IO::Socket::INET>, L<IO::Socket>, L<IO::Handle>
 
 =head1 AUTHOR
 
-Shawn M Moore, C<< <sartak at gmail.com> >>
-
-=head1 BUGS
-
-No known bugs.
-
-Please report any bugs through RT: email
-C<bug-io-socket-telnet at rt.cpan.org>, or browse to
-L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=IO-Socket-Telnet>.
-
-=head1 SUPPORT
-
-You can find this documentation for this module with the perldoc command.
-
-    perldoc IO::Socket::Telnet
-
-You can also look for information at:
-
-=over 4
-
-=item * AnnoCPAN: Annotated CPAN documentation
-
-L<http://annocpan.org/dist/IO-Socket-Telnet>
-
-=item * CPAN Ratings
-
-L<http://cpanratings.perl.org/d/IO-Socket-Telnet>
-
-=item * RT: CPAN's request tracker
-
-L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=IO-Socket-Telnet>
-
-=item * Search CPAN
-
-L<http://search.cpan.org/dist/IO-Socket-Telnet>
-
-=back
+Shawn M Moore, C<sartak@gmail.com>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2007 Shawn M Moore.
+Copyright 2007, 2009 Shawn M Moore.
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
 
 =cut
 
-1;
-
